#include "dialog.h"
#include "ui_dialog.h"
#include <QTextCodec>

void UCS2toUTF8(unsigned short *ucs2, int count, char *utf8)
{
    unsigned short unicode;
    unsigned char bytes[4] = {0};
    int nbytes = 0;
    int i = 0, j = 0;
    int len=0;

    if((ucs2 != NULL) && (utf8 != NULL))
    {
        if(count  == 0)
        {
            len = 0;
        }
        else
        {
           for (i=0; i<count; i++)
           {
                unicode = ucs2[i];

                if (unicode < 0x80)
                {
                 nbytes = 1;
                 bytes[0] = unicode;
                }
                else if (unicode < 0x800)
                {
                 nbytes = 2;
                 bytes[1] = (unicode & 0x3f) | 0x80;
                                bytes[0] = (((unicode << 2) & 0x1f00) | 0xc000) >> 8;
                }
                else
                {
                 nbytes = 3;
                 bytes[2] = (unicode & 0x3f) | 0x80;
                 bytes[1] = (((unicode << 2) & 0x3f00) | 0x8000) >> 8;
                                bytes[0] = (((unicode << 4) & 0x0f0000) | 0xe00000) >> 16;
                }

                for (j=0; j<nbytes; j++)
                {
                    utf8[len] = bytes[j];
                    len++;
                }
            }
        }
        utf8[len] = '\0';
    }
}

const char voiceList[8][32] = {
    "请放卡",
    "读卡成功",
    "读卡成功，请重新放卡",
    "读卡成功，请核验指纹",
    "核验成功",
    "核验失败，请重按手指",
    "写卡成功",
    "写卡失败"
};


Dialog::Dialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::Dialog)
{

#if (QT_VERSION <= QT_VERSION_CHECK(5,0,0))
#if _MSC_VER
    QTextCodec *codec = QTextCodec::codecForName("gbk");
#else
    QTextCodec *codec = QTextCodec::codecForName("utf-8");
#endif
    QTextCodec::setCodecForLocale(codec);
    QTextCodec::setCodecForCStrings(codec);
    QTextCodec::setCodecForTr(codec);
#else
    QTextCodec *codec = QTextCodec::codecForName("utf-8");
    QTextCodec::setCodecForLocale(codec);
#endif
    ui->setupUi(this);
    for (int i=0;i<8;i++) {
        ui->cmbVoiceList->addItem(voiceList[i]);
    }
}

Dialog::~Dialog()
{
    delete ui;
}


char* Dialog::reverse( char *s, int left, int right )
{
    if(left >= right)
        return s ;
    char t = s[left] ;
    s[left] = s[right] ;
    s[right] = t ;
    return reverse(s, left + 1, right - 1) ;
}

QString Dialog::decodeGAT(unsigned char* wzText)
{
    QString strResult;
    QString strLicTypeText = "证件类型: 港澳台居住证.\r\n";
    strResult += strLicTypeText;
    unsigned short szTemp[256] = {0x0};
    char szUtf8[512] = {0x0};
    memcpy(szTemp, wzText, 15*2);
    UCS2toUTF8(szTemp, 15, szUtf8);
    QString strNameText;
    strNameText.sprintf("姓名：%s, ", szUtf8);
    strResult += strNameText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+15*sizeof(unsigned short), 1*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 1, szUtf8);
    QString strSexText;
    strSexText.sprintf("性别：%s, ", szUtf8);
    strResult += strSexText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+18*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strBornText;
    strBornText.sprintf("出生日期：%s, ", szUtf8);
    strResult += strBornText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+26*sizeof(unsigned short), 35*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 35, szUtf8);
    QString strAddressText;
    strAddressText.sprintf("常住地址：%s, ", szUtf8);
    strResult += strAddressText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+61*sizeof(unsigned short), 18*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 18, szUtf8);
    QString strLicIDText;
    strLicIDText.sprintf("身份证号：%s, ", szUtf8);
    strResult += strLicIDText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+79*sizeof(unsigned short), 15*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 15, szUtf8);
    QString strIssueAtText;
    strIssueAtText.sprintf("签发机关：%s, ", szUtf8);
    strResult += strIssueAtText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+94*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strEfftiveDateText;
    strEfftiveDateText.sprintf("起始日期：%s, ", szUtf8);
    strResult += strEfftiveDateText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+102*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strExpireDateText;
    strExpireDateText.sprintf("截止日期：%s, ", szUtf8);
    strResult += strExpireDateText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+110*sizeof(unsigned short), 9*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 9, szUtf8);
    QString strPassNumText;
    strPassNumText.sprintf("通行证号码：%s, ", szUtf8);
    strResult += strPassNumText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+119*sizeof(unsigned short), 2*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 2, szUtf8);
    QString strIssueNumText;
    strIssueNumText.sprintf("签证次数：%s\r\n", szUtf8);
    strResult += strIssueNumText;
    return strResult;
}

QString Dialog::decodeCN(unsigned char* wzText)
{
    QString strResult;
    QString strLicTypeText = ("证件类型: 居民身份证.\r\n");
    strResult += strLicTypeText;
    unsigned short szTemp[256] = {0x0};
    char szUtf8[512] = {0x0};
    memcpy(szTemp, wzText, 15*2);
    UCS2toUTF8(szTemp, 15, szUtf8);
    QString strNameText;
    strNameText.sprintf("姓名：%s, ", szUtf8);
    strResult += strNameText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+15*sizeof(unsigned short), 1*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 1, szUtf8);
    QString strSexText;
    strSexText.sprintf("性别：%s, ", szUtf8);
    strResult += strSexText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+16*sizeof(unsigned short), 2*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 2, szUtf8);
    QString strNationText;
    strNationText.sprintf("民族：%s, ", szUtf8);
    strResult += strNationText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+18*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strBornText;
    strBornText.sprintf("出生日期：%s, ", szUtf8);
    strResult += strBornText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+26*sizeof(unsigned short), 35*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 35, szUtf8);
    QString strAddressText;
    strAddressText.sprintf("常住地址：%s, ", szUtf8);
    strResult += strAddressText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+61*sizeof(unsigned short), 18*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 18, szUtf8);
    QString strLicIDText;
    strLicIDText.sprintf("身份证号：%s, ", szUtf8);
    strResult += strLicIDText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+79*sizeof(unsigned short), 15*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 15, szUtf8);
    QString strIssueAtText;
    strIssueAtText.sprintf("签发机关：%s, ", szUtf8);
    strResult += strIssueAtText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+94*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strEfftiveDateText;
    strEfftiveDateText.sprintf("起始日期：%s, ", szUtf8);
    strResult += strEfftiveDateText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+102*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strExpireDateText;
    strExpireDateText.sprintf("截止日期：%s, ", szUtf8);
    strResult += strExpireDateText;

    return strResult;
}

QString Dialog::decodePRP(unsigned char* wzText)
{
    QString strResult;
    unsigned short szTemp[256] = {0x0};
    char szUtf8[512] = {0x0};
    QString strLicTypeText = ("证件类型: 外国人永久居住证.\r\n");
    strResult += strLicTypeText;
    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText, 60*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 60, szUtf8);
    QString strEnNameText;
    strEnNameText.sprintf("英文名：%s, ", szUtf8);
    strResult += strEnNameText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+60*sizeof(unsigned short), 1*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 1, szUtf8);
    QString strSexText;
    strSexText.sprintf("性别：%s, ", szUtf8);
    strResult += strSexText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+61*sizeof(unsigned short), 15*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 15, szUtf8);
    QString strLicIDText;
    strLicIDText.sprintf("证件号：%s, ", szUtf8);
    strResult += strLicIDText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+76*sizeof(unsigned short), 3*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 3, szUtf8);
    QString strCountryCode;
    strCountryCode.sprintf("国家代码：%s, ", szUtf8);
    strResult += strCountryCode;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+79*sizeof(unsigned short), 15*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 15, szUtf8);
    QString strCnName;
    strCnName.sprintf("中文名：%s, ", szUtf8);
    strResult += strCnName;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+94*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strEffectiveDateText;
    strEffectiveDateText.sprintf("签发日期：%s, ", szUtf8);
    strResult += strEffectiveDateText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+102*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strExpireDateText;
    strExpireDateText.sprintf("截止日期：%s, ", szUtf8);
    strResult += strExpireDateText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+110*sizeof(unsigned short), 8*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 8, szUtf8);
    QString strBornText;
    strBornText.sprintf("出生日期：%s, ", szUtf8);
    strResult += strBornText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+118*sizeof(unsigned short), 2*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 2, szUtf8);
    QString strLicVerText;
    strLicVerText.sprintf("证件版本：%s, ", szUtf8);
    strResult += strLicVerText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+120*sizeof(unsigned short), 4*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 4, szUtf8);
    QString strIssueAtCodeText;
    strIssueAtCodeText.sprintf("签发机构代码：%s, ", szUtf8);
    strResult += strIssueAtCodeText;

    memset(szUtf8, 0x0, 512);
    memcpy(szTemp, wzText+124*sizeof(unsigned short), 1*sizeof(unsigned short));
    UCS2toUTF8(szTemp, 1, szUtf8);
    QString strLicFlagText;
    strLicFlagText.sprintf("证件标识：%s\r\n", szUtf8);
    strResult += strLicFlagText;
    return strResult;
}



void Dialog::on_bnEnum_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    ui->cmbDevices->clear();
    //SDT_ConfigLog(X_LOG_DEBUG,X_LOG_TYPE_FILE,(char*)"zklog.txt");
    SDT_ConfigLog(X_LOG_DEBUG,X_LOG_TYPE_CONSOLE,(char*)"zklog.txt");
    m_deviceCount = SDT_EnumDevice(m_deviceList, 16);
    if (m_deviceCount <= 0)
    {
        ui->textBase->setText("找不到设备");
        return;
    }
    for (int i=0;i<m_deviceCount;i++) {
        QString itemStr = QString("%1_%2_%3").arg(m_deviceList[i].vid).arg(m_deviceList[i].pid).arg(m_deviceList[i].device_address);
        ui->cmbDevices->addItem(itemStr);
    }
}

void Dialog::on_bnOpen_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL != m_hDevice)
    {
        ui->textBase->setText("设备已打开!");
        return;
    }
    if (ui->checkUSB->isChecked())
    {
        int curSelected = ui->cmbDevices->currentIndex();
        if (curSelected == -1)
        {
            ui->textBase->setText("请先选择设备!");
            return;
        }
        m_hDevice = SDT_OpenDevice(&m_deviceList[curSelected]);
    }
    else if( ui->checkSERIAL->isChecked())
    {
        QString serialName = ui->edSerialName->text();
        if (serialName.isEmpty())
        {
            ui->textBase->setText("请请选择设备/输入串口节点名(如:/dev/ttyUSB0等)!");
            return;
        }
        //SDT_ConfigLog(X_LOG_DEBUG,X_LOG_TYPE_FILE,(char*)"zklog.txt");
        SDT_ConfigLog(X_LOG_DEBUG,X_LOG_TYPE_CONSOLE,(char*)"zklog.txt");
        m_hDevice = SDT_OpenDevice2(NULL, serialName.toStdString().c_str(), 115200);
    }
    else
    {
        ui->textBase->setText("请选择设备数据通信方式!");
        return;
    }
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("设备打开失败!");
        return;
    }
    m_isSerialConnect = !(ui->checkUSB->isChecked());
    ui->textBase->setText("连接成功");
}

void Dialog::on_bnGetSAMID_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("请先打开设备!");
        return;
    }
    char szSamid[64] = {0x0};
    int ret = SDT_GetSAMIDToStr(m_hDevice, szSamid);
    if (0x90 == ret)
    {
        QString strText = QString("SAMID:%1").arg(szSamid);
        ui->textBase->setText(strText);
    }
    else {
        QString strText;
        strText.sprintf("读取失败，错误码：%02X",ret);
        ui->textBase->setText(strText);
    }

}

void Dialog::on_bnStatus_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("请先打开设备!");
        return;
    }
    int ret = SDT_GetSAMStatus(m_hDevice);
    if (0x90 == ret)
    {
        ui->textBase->setText("状态正常!");
    }
    else {
        QString strText;
        strText.sprintf("状态异常，错误码：%02X",ret);
        ui->textBase->setText(strText);
    }
}

void Dialog::on_bnReset_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("请先打开设备!");
        return;
    }
    int ret = SDT_GetSAMStatus(m_hDevice);
    if (0x90 == ret)
    {
        ui->textBase->setText("重置SAM_A模块成功!");
    }
    else {
        QString strText;
        strText.sprintf("重置SAM_A模块失败，错误码：%02X",ret);
        ui->textBase->setText(strText);
    }
}

void Dialog::on_bnReadCard_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("请先打开设备!");
        return;
    }
    unsigned char bufTmp[256] = {0x0};
    unsigned char wzText[256] = {0x0};
    unsigned char wltData[1024] = {0x0};
    unsigned char fpData[1024] = {0x0};
    unsigned int wzLen = 0;
    unsigned int wltLen = 0;
    unsigned int fpLen = 0;
    if (0x9F != SDT_StartFindIDCard(m_hDevice, bufTmp))
    {
        ui->textBase->setText("寻卡失败!");
        ui->label_HeadPhoto->clear();
        return;
    }
    if (0x90 != SDT_SelectIDCard(m_hDevice, bufTmp))
    {
        ui->textBase->setText("选卡失败!");
        ui->label_HeadPhoto->clear();
        return;
    }
    if (0x90 != SDT_ReadBaseFPMsg(m_hDevice, wzText, &wzLen, wltData, &wltLen, fpData, &fpLen))
    {
        ui->textBase->setText("读卡失败!");
        ui->label_HeadPhoto->clear();
        return;
    }

    QString baseInfoText;
    unsigned short szTemp[256] = {0x0};
    char szUtf8[512] = {0x0};
    memcpy(szTemp, wzText+248, 2);
    UCS2toUTF8(szTemp, 1, szUtf8);
    if (strcmp(szUtf8, "I") == 0)
    {
        baseInfoText = decodePRP(wzText);
    }
    else if (strcmp(szUtf8, "J") == 0)
    {
        baseInfoText = decodeGAT(wzText);
    }
    else
    {
        baseInfoText = decodeCN(wzText);
    }
    ui->textBase->setText(baseInfoText);

    if (wltLen > 0)
    {
        QByteArray ba((char*)wltData, wzLen);
        ui->textWlt->setText(ba.toHex());

        int ret = 0;
        unsigned char bgr[102*126*3]={0x0};
        ret =  SDT_Wlt2Bgr24((unsigned char*)wltData,bgr,102*126*3);
        if(ret != 1)
        {
            ui->textBase->append(QString("照片解码失败! 错误码:%1").arg(ret));
            ui->label_HeadPhoto->clear();
        }
        else
        {
            int i = 0;
            int nPos = 0;
            char imageData [1024*1024]={0x0};
            for (i=0;i<102*126;i++)
            {
                imageData[nPos++] = bgr[i*3+2];
                imageData[nPos++] = bgr[i*3+1];
                imageData[nPos++] = bgr[i*3];
                if ((i+1)%102 == 0)
                {
                    imageData[nPos++] = 0xFF;
                    imageData[nPos++] = 0xFF;
                }
            }
            reverse( imageData, 0, nPos );
            QImage image((const uchar*)imageData, 102, 126, QImage::Format_RGB888);
            image = image.mirrored(true,false);

#if 1    //delete photo back ground color
            image = image.convertToFormat(QImage::Format_ARGB32);
            union myrgb
            {
                uint rgba;
                uchar rgba_bits[4];
            };
            myrgb* mybits =(myrgb*) image.bits();
            int len = image.width()*image.height();
            while(len --> 0)
            {
                mybits->rgba_bits[3] = (  mybits->rgba==0xFFFAFAFA
                                ||mybits->rgba==0xFFFAFFFF
                                ||mybits->rgba==0xFFFBFBFB
                                ||mybits->rgba==0xFFFBFFF
                                ||mybits->rgba==0xFFFCFCFC
                                ||mybits->rgba==0xFFFCFFFF
                                ||mybits->rgba==0xFFFDFDFD
                                ||mybits->rgba==0xFFFDFFFF
                                ||mybits->rgba==0xFFFEFEFE
                                ||mybits->rgba==0xFFFEFFFF
                                ||mybits->rgba==0xFFFFFFFF )?0:255;
                mybits++;
            }
#endif
            ui->label_HeadPhoto->setPixmap(QPixmap::fromImage(image).scaled( ui->label_HeadPhoto->width(),ui->label_HeadPhoto->height()));
        }
    }
    if (fpLen > 0)
    {
        QByteArray ba((char*)fpData, fpLen);
        ui->textFPData->setText(ba.toHex());
    }
}

void Dialog::on_bnClose_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("请先打开设备!");
        return;
    }
    SDT_CloseDevice(m_hDevice);
    m_hDevice = NULL;
    ui->textBase->setText("设备关闭成功!");
}

void Dialog::on_bnIDSnr_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("请先打开设备!");
        return;
    }
    if (!m_isSerialConnect)
    {
        ui->textBase->setText("仅支持串口连接");
        return;
    }
    char szIDSnr[64] = {0x0};
    int ret = ZKMF_ReadIDCardSnr(m_hDevice, szIDSnr);
    if (0 != ret)
    {
        ui->textBase->setText("操作失败");
    }
    else
    {
        QString strText;
        strText.sprintf("身份证物理卡号：%s", szIDSnr);
        ui->textBase->setText(strText);
    }
}

void Dialog::on_bnICSnr_clicked()
{
    ui->textBase->setText("");
    ui->textWlt->setText("");
    ui->textFPData->setText("");
    if (NULL == m_hDevice)
    {
        ui->textBase->setText("请先打开设备!");
        return;
    }
    if (!m_isSerialConnect)
    {
        ui->textBase->setText("仅支持串口连接");
        return;
    }
    unsigned int icSnr = 0;
    int ret = ZKMF_ReadICCardSnr(m_hDevice, 0x26, 0, &icSnr);
    if (0 != ret)
    {
        ui->textBase->setText("操作失败");
    }
    else
    {
        QString strText;
        strText.sprintf("IC物理卡号：%u", icSnr);
        ui->textBase->setText(strText);
    }
}

void Dialog::on_bnPlayVoice_clicked()
{
    if (!ISSupportHIDVoice())
    {
        ui->textBase->setText("操作失败");
        return;
    }
    int curSelected = ui->cmbVoiceList->currentIndex();
    if (curSelected == -1)
    {
        ui->textBase->setText("请先选择语音!");
        return;
    }
    int ret = ZKIDVoice(curSelected);
    if (1 == ret)
    {
        ui->textBase->setText("操作成功");
        return;
    }
    else
    {
        ui->textBase->setText("操作失败");
        return;
    }
}
