#ifndef LIBSDTAPI_H
#define LIBSDTAPI_H
#include "xusbdevicedef.h"
#ifdef __ANDROID__
#include <jni.h>
#endif

#ifndef APICALL
#ifdef _WIN32
#define APICALL __APICALL
#else
#define APICALL
#endif
#endif



typedef enum X_LOG_LEVEL {
   X_LOG_VERBOSE = 2,
   X_LOG_DEBUG = 3,
   X_LOG_INFO = 4,
   X_LOG_WARN = 5,
   X_LOG_ERROR = 6,
   X_LOG_ASSERT = 7,
} X_Log_Level;

typedef enum X_LOG_TYPE{
    X_LOG_TYPE_CONSOLE		=	1,	//terminal
    X_LOG_TYPE_FILE		=	2,
    X_LOG_TYPE_DBGVIEW		=	3,	//windows only
}X_Log_Type;



#ifdef __cplusplus
extern "C"
{
#endif
    void APICALL SDT_SetLogLevel(int nLevel);
    void APICALL SDT_ConfigLog(int nLevel, int nType, char* fileName);
    int APICALL SDT_EnumDevice(TXUSBDevice deviceList[], int maxCount);
    void* APICALL SDT_OpenDevice(PXUSBDevice pXUSBDevice);
    void* APICALL SDT_OpenDevice2(PXUSBDevice pXUSBDevice, const char* pSerialName, int baudRate);
    #ifdef __ANDROID__
	void* APICALL SDT_OpenDeviceByAndroidHost(JavaVM * jvm, JNIEnv *env, jobject obj);
	void* APICALL SDT_OpenDeviceByFD(int vid, int pid, int busNum, int dev_address, int fd, int inEndpoint, int outEndpoint);
#endif
#ifdef __ANDROID__
	int APICALL SDT_CloseDeviceByAndroidHost(JavaVM * jvm, JNIEnv *env, void *handle);
#endif

	int APICALL SDT_CloseDevice(void *handle);
    int APICALL SDT_ResetSAM(void* handle);
    int APICALL SDT_GetSAMStatus(void* handle);
    int APICALL SDT_GetSAMID(void* handle, unsigned char * pucSAMID);
    int APICALL SDT_GetSAMIDToStr(void* handle, char *pcSAMID);
    int APICALL SDT_StartFindIDCard(void* handle, unsigned char *pucManaInfo);
    int APICALL SDT_SelectIDCard(void* handle, unsigned char *pucManaMsg);
    int APICALL SDT_ReadBaseMsg(void* handle, unsigned char *pucCHMsg, unsigned int *puiCHMsgLen, unsigned char *pucPHMsg, unsigned int *puiPHMsgLen);
    int APICALL SDT_ReadBaseFPMsg(void* handle, unsigned char *pucCHMsg, unsigned int *puiCHMsgLen, unsigned char *pucPHMsg, unsigned int *puiPHMsgLen, unsigned char *pucFPMsg, unsigned int *puiFPMsgLen);
    int APICALL SDT_Wlt2Bgr24(unsigned char* wltData,unsigned char*bgrData,int cbBgrData) ;
    int APICALL SDT_Wlt2BmpBuffer(unsigned char* wltData,unsigned char*imageData,int cbImageData);
    
    //中控接口
    int APICALL ZKMF_SetDevNum(void* handle, const unsigned char *snr);
    int APICALL ZKMF_GetDevNum(void* handle, unsigned char *addr, unsigned char *snr);
    int APICALL ZKMF_GetDevVersionNum(void* handle, char *VerNum);
    int APICALL ZKMF_FindCard(void* handle, unsigned char mode, unsigned short *type);
    int APICALL ZKMF_AntiCard(void* handle, unsigned char *flag, unsigned int *coreNum);
    int APICALL ZKMF_SelectCard(void* handle, unsigned int coreNum, unsigned int *coreNumRep);
    int APICALL ZKMF_Halt(void* handle);
    int APICALL ZKMF_RDY_Read(
        void* handle, 
        unsigned char requestmode,
        unsigned char keymode,
        unsigned char blkNum,
        unsigned char addr,
        const char *key,
        unsigned int *cardNum,
        unsigned char *data,
        unsigned int  *dataLen
        );
        

    int APICALL ZKMF_RDY_Write(
            void* handle, 
            unsigned char requestmode,
            unsigned char keymode,
            unsigned char blkNum,
            unsigned char addr,
            const char *key,
            const unsigned char *data,
            unsigned int dataLen,
            unsigned int *cardNum
            );

    int APICALL ZKMF_ReadICCardSnr(void* handle, unsigned char mode, unsigned char bHalt, unsigned int *coreNum);
    int APICALL ZKMF_ReadIDCardSnr(void* handle, char *Snr);

    int APICALL ZKMF_ReadIDCardDNCode(void* handle, char* dnCode);
    int APICALL ZKMF_ReadBankCardNum(void* handle, char* cardNum, int size);

#ifdef __cplusplus
};
#endif

#endif
